import {select, transition, scaleLinear, min, max, extent} from 'd3';

export const renderBarChart = () => {
	
  const sampleData = [20, 30, 40, 50, 80, 79, 29, 23, 45];
  
  const barHeight = 20;
  
  // Selecting on #canvas2
  const canvas = select('#canvas2');
  
  // Get #canvas2 width & height
  const canvasWidth = +canvas.attr('width');
  
  const canvasHeight = +canvas.attr('height');
  
 
  // Magin Offset
  const xOffset = 80;
  
  const yOffset = 50;
	
  // Scale factor (to be removed)
  const scaleFactor = 5;
  
  // Set background color change after 5seconds
  const timer = transition()
  	.duration(2000);
  
  canvas.transition(timer)
    .style('background-color', '#E4DC3D'); // light green '#E4DC3D'
  
  // Create d3 update selection
  const gUpdate = canvas
  	.selectAll('g')
  	.data(sampleData);
  
  // Create d3 enter() selection
  // Create a list of svg groups each (<g></g>) for one data dimension 
  // in ${data} and 
  // set coordinate of each <g></g> at (0, `${i * barHeight}`)
  const gEnter = gUpdate
  	.enter()
  	.append('g')
  	.attr('transform', (d, i) => {
    	return 'translate(0,' + i * barHeight + ')';
    });
 
  // Add rectangle for each data dimension
  // with virtual scale factor 
  // (todo: this should be done with xScale)
  // gEnter
  //   .append('rect')
  // 	.attr('x', xOffset)
  // 	.attr('y', yOffset)
  // 	.attr('width', d => d * scaleFactor)
  // 	.attr('height', barHeight - 2)
  // 	.attr('fill', '#CE883D');
  
  // Add text label for each bar
  // gEnter
  // 	.append('text')
  // 	.attr('class', 'bar-text')
  // 	.attr('x', d => xOffset + d * scaleFactor)
  // 	.attr('y', yOffset + barHeight/2)
  // 	.text(d => d);
  
  
  // Get first canvas position, width & height
  const canvas1_xpos = select('#canvas').attr('x');
  const canvas1_ypos = select('#canvas').attr('y');
  const canvas1_width = select('#canvas').attr('width');
  const canvas1_height = select('#canvas').attr('height');
  
  const canvas2_width = select('#canvas2').attr('width');
  const canvas2_height = select('#canvas2').attr('height');
  
  // Set #canvas2 current position (to align on first canvas position)
  const canvas2_xpos = canvas2_width;//canvas1_width;//canvas1_xpos + canvas1_width;//canvas.attr('x');
  const canvas2_ypos = -1 * canvas2_height;//canvas.attr('y');
  
  // NOTE: each SVG canvas has its own relative coordinate origin (0, 0),
  // You can align canvas2 at right side of first canvas horizontally
  // by adding .attr('transform', 'translate(' + canvas2_xpos + ',' + canvas2_ypos + ' )')
  // at below code!!
	// Add Bar chart header 
  canvas
  	// .attr('transform', 'translate(' + canvas2_xpos + ',' + canvas2_ypos + ' )')
  	.append('text')
  	.attr('class', 'title-text')
    .attr('x', canvasWidth/2 - 50)
  	.attr('y', yOffset / 2)
  	.text('Bar chart')
  	;
  
	///////////////////
  // Scale Linear  //
  ///////////////////
  // Constructs a continuous linear scale where we can input data (domain) 
  // maps to the specified output range
  // Scaling sampleData dimension onto canvas width
  const xScale = scaleLinear()
  	.domain([0, max(sampleData, d => d)]) // working perfect
  	// .domain(extent(sampleData, d => d)) // not working appropriately
  	.range([0, canvasWidth - xOffset * 2])
  	.nice();
  
  // Add color panel
  // const colorPanel = d3.scaleOrdinal([
  //   'gray', 'green', 'brown', 'orange', 'red', 'purple'
  // ]);
  const colorPanel = d3.scaleOrdinal([
    'gray', 'green', '#BA53F4', '#B8473D', '#3A473D', '#CE883D'
  ]);
  
  // Add rectangle for each data dimension
  // scaling data dimension with canvas width
  // NOTE: (x,y) should set to (xOffset, yOffset) to align with together
  gEnter
    .append('rect')
  	.attr('x', xOffset)
  	.attr('y', yOffset)
  	.attr('width', d => xScale(d)) 
  	.attr('height', barHeight-1)
  	.attr('fill', d => colorPanel(d));//'#CE883D'
  
  // Add text label for each bar
  // scaling data dimension with canvas width
  // NOTE: (x,y) should add offset (xOffset, yOffset) to align with together
  gEnter
  	.append('text')
  	.attr('class', 'bar-text')
  	.attr('x', d =>  xOffset + xScale(d)) 
  	.attr('y', yOffset + barHeight/2)
    .attr("dy", ".15em")
  	.text(d => d);
  
  // Get X-Axis Scale
  const xAxis = d3.axisBottom().scale(xScale);
  // Get X-Axis X & Y position
  const x_axis_x_pos = xOffset;
  const x_axis_y_pos = sampleData.length * barHeight + yOffset;
  
  // Add X-Axis into #canvas2
  canvas
  	.append('g')
  	.attr('transform', 'translate('+ x_axis_x_pos + ','  + x_axis_y_pos + ')')
    .call(xAxis);
  
  
  
}